@@ -26,12 +26,12 @@
 #include <time.h>
 
 #include <glib.h>
-#include <gnome.h>
 #include <libgnomevfs/gnome-vfs-types.h>
 #include <libgnomevfs/gnome-vfs-mime.h>
 
 #include "file-data.h"
 #include "file-utils.h"
+#include "glib-utils.h"
 #include "fr-command.h"
 #include "fr-command-7z.h"
 
@@ -88,20 +88,24 @@ change_to_unix_dir_separator (char *path)
 	char *c;
 	
 	for (c = path; *c != 0; c++)
-		if (*c == '\\')
+		if ((*c == '\\') && (*(c+1) != '\\'))
 			*c = '/';
 }
 
 
-static void
-eat_trailing_spaces (char *name)
+static char*
+to_dos (const char *path)
 {
-	char *c = name + strlen (name) - 1;
+	return str_substitute (path, "/", "\\\\");
+}
 
-	while ((*c == ' ') && (c != name)) {
-		*c = 0;
-		c--;
-	}
+
+static int
+str_rfind (const char *str, 
+	   int         c)
+{
+	char *tmp = strrchr (str, c);
+	return (tmp == NULL) ? -1 : (tmp - str);
 }
 
 
@@ -119,6 +123,7 @@ list__process_line (char     *line,
 
 	if (! p7z_comm->list_started) {
 		if (strncmp (line, "--------", 8) == 0) {
+			p7z_comm->name_index = str_rfind (line, ' ') + 1;
 			p7z_comm->list_started = TRUE;
 		}
 		return;
@@ -131,7 +136,13 @@ list__process_line (char     *line,
 
 	fdata = file_data_new ();
 
-	fields = split_line (line, 5);
+	fields = split_line (line, 4);
+
+	if (g_strv_length (fields) < 4) {
+		g_strfreev (fields);
+		file_data_free (fdata);
+		return;
+	}
 
 	if (fields[2][0] == 'D') { /* skip directories */
 		g_strfreev (fields);
@@ -139,12 +150,11 @@ list__process_line (char     *line,
 		return;
 	}
 
-	fdata->size = atol (fields[3]);
+	fdata->size = g_ascii_strtoull (fields[3], NULL, 10);
 	fdata->modified = mktime_from_string (fields[0], fields[1]); 
 	g_strfreev (fields);
 
-	name_field = g_strdup (get_last_field (line, 6));
-	eat_trailing_spaces (name_field);
+	name_field = g_strdup (line + p7z_comm->name_index);
 
 	change_to_unix_dir_separator (name_field);
 	
@@ -170,14 +180,10 @@ list__process_line (char     *line,
 static void
 fr_command_7z_begin_command (FRCommand *comm)
 {
-	char *path;
-
-	path = g_find_program_in_path ("7za");
-	if (path != NULL)
+	if (is_program_in_path ("7za"))
 		fr_process_begin_command (comm->process, "7za");
 	else
 		fr_process_begin_command (comm->process, "7z");
-	g_free (path);
 }
 
 
@@ -235,7 +241,7 @@ fr_command_7z_add (FRCommand     *comm,
 	fr_process_add_arg (comm->process, comm->e_filename);
 
 	for (scan = file_list; scan; scan = scan->next) {
-		char *filename = str_substitute ((char*) scan->data, "/", "\\\\");
+		char *filename = to_dos (scan->data);
 		fr_process_add_arg (comm->process, filename);
 		g_free (filename);
 	}
@@ -258,7 +264,7 @@ fr_command_7z_delete (FRCommand *comm,
 	fr_process_add_arg (comm->process, comm->e_filename);
 
 	for (scan = file_list; scan; scan = scan->next) {
-		char *filename = str_substitute ((char*) scan->data, "/", "\\\\");
+		char *filename = to_dos (scan->data);
 		fr_process_add_arg (comm->process, filename);
 		g_free (filename);
 	}
@@ -299,7 +305,7 @@ fr_command_7z_extract (FRCommand  *comm,
 	fr_process_add_arg (comm->process, comm->e_filename);
 
 	for (scan = file_list; scan; scan = scan->next) {
-		char *filename = str_substitute ((char*) scan->data, "/", "\\\\");
+		char *filename = to_dos (scan->data);
 		fr_process_add_arg (comm->process, filename);
 		g_free (filename);
 	}
@@ -343,6 +349,8 @@ fr_command_7z_class_init (FRCommand7zClass *class)
 static void 
 fr_command_7z_init (FRCommand *comm)
 {
+	comm->file_type = FR_FILE_TYPE_7ZIP;
+
 	comm->propAddCanUpdate             = TRUE; 
 	comm->propAddCanReplace            = TRUE; 
 	comm->propExtractCanAvoidOverwrite = FALSE;
@@ -399,6 +407,11 @@ fr_command_7z_new (FRProcess  *process,
 {
 	FRCommand *comm;
 
+	if ((!is_program_in_path("7za")) &&
+	    (!is_program_in_path("7z"))) {
+		return NULL;
+	}
+
 	comm = FR_COMMAND (g_object_new (FR_TYPE_COMMAND_7Z, NULL));
 	fr_command_construct (comm, process, filename);
 
