@@ -31,6 +31,7 @@
 
 #include "file-data.h"
 #include "file-utils.h"
+#include "glib-utils.h"
 #include "fr-command.h"
 #include "fr-command-zip.h"
 
@@ -91,6 +92,23 @@ mktime_from_string (char *date_s,
 }
 
 
+static char *
+fr_command_zip_escape (FRCommand     *comm,
+		       const char    *str)
+{
+	char *estr;
+	char *estr2;
+
+	estr = escape_str (str, "\\");
+	estr2 = shell_escape (estr);
+
+	g_free (estr);
+
+	return estr2;
+	
+}
+
+
 static char*
 zip_escape (const char *str)
 {
@@ -155,7 +173,7 @@ list__process_line (char     *line,
 	fdata = file_data_new ();
 
 	fields = split_line (line, 7);
-	fdata->size = atoll (fields[0]);
+	fdata->size = g_ascii_strtoull (fields[0], NULL, 10);
 	fdata->modified = mktime_from_string (fields[4], fields[5]);
 	g_strfreev (fields);
 
@@ -204,7 +222,7 @@ add_password_arg (FRCommand     *comm,
 
 		fr_process_add_arg (comm->process, "-P");
 
-		e_password = escape_str (password, "\"");
+		e_password = escape_str (password, "\"*?[]'`()$!;");
 		if (e_password != NULL) {
 			arg = g_strconcat ("\"", e_password, "\"", NULL);
 			g_free (e_password);
@@ -248,10 +266,9 @@ process_line__common (char     *line,
 	fr_command_message (comm, line);
 
 	if (comm->n_files != 0) {
-		double fraction = (double) comm->n_file++ / comm->n_files;
+		double fraction = (double) comm->n_file++ / (comm->n_files+1);
 		fr_command_progress (comm, fraction);
 	}
-
 }
 
 
@@ -420,12 +437,16 @@ fr_command_zip_class_init (FRCommandZipClass *class)
 	afc->extract        = fr_command_zip_extract;
 	afc->test           = fr_command_zip_test;
 	afc->handle_error   = fr_command_zip_handle_error;
+
+	afc->escape         = fr_command_zip_escape;
 }
 
  
 static void 
 fr_command_zip_init (FRCommand *comm)
 {
+	comm->file_type = FR_FILE_TYPE_ZIP;
+
 	comm->propAddCanUpdate             = TRUE; 
 	comm->propAddCanReplace            = TRUE; 
 	comm->propExtractCanAvoidOverwrite = TRUE;
@@ -484,8 +505,8 @@ fr_command_zip_new (FRProcess  *process,
 {
 	FRCommand *comm;
 
-	if ((!is_program_in_path("zip")) ||
-	    (!is_program_in_path("unzip"))) {
+	if (!is_program_in_path ("zip") &&
+	    !is_program_in_path ("unzip")) {
 		return NULL;
 	}
 
