@@ -24,12 +24,13 @@
 #define FR_WINDOW_H
 
 #include <gtk/gtk.h>
+#include <libgnomevfs/gnome-vfs-mime-handlers.h>
 #include "egg-recent.h"
 #include "fr-archive.h"
 #include "typedefs.h"
 #include "file-list.h"
 
-#define GCONF_NOTIFICATIONS 8
+#define GCONF_NOTIFICATIONS 9
 
 enum {
 	COLUMN_FILE_DATA,
@@ -76,6 +77,7 @@ typedef struct {
 	GtkListStore *   list_store;
 	GtkWidget *      toolbar;
 	GtkWidget *      statusbar;
+	GtkWidget *      progress_bar;
 	GtkWidget *      location_bar;
 	GtkWidget *      location_entry;
 	GtkWidget *      location_label;
@@ -83,6 +85,10 @@ typedef struct {
 	GtkWidget *      home_button;
 	GtkWidget *      back_button;
 	GtkWidget *      fwd_button;
+	GtkCellRenderer *name_renderer;
+	GtkTreePath     *hover_path;
+
+	gint             current_view_length;
 
 	GtkTooltips     *tooltips;
 	guint            help_message_cid;
@@ -93,6 +99,7 @@ typedef struct {
 	GtkWidget *      down_arrows[5];
 
 	FRArchive *      archive;
+	FRAction         current_action;
 	gboolean         archive_present;
 	gboolean         archive_new;        /* A new archive has been created
 					      * but it doesn't contain any 
@@ -108,11 +115,14 @@ typedef struct {
 					       * in the Add dialog. */
 	char *           extract_default_dir; /* default directory to be used
 					       * in the Extract dialog. */
+	gboolean         freeze_default_dir;
 
 	gboolean         view_folder_after_extraction;
 	char *           folder_to_view;
 
 	gboolean         give_focus_to_the_list;
+	gboolean         single_click;
+	GtkTreePath     *path_clicked;
 
 	WindowSortMethod sort_method;
 	GtkSortType      sort_type;
@@ -143,10 +153,13 @@ typedef struct {
 
 	GtkActionGroup  *actions;
 
-	EggRecentViewUIManager *recent_view;
+	EggRecentViewGtk *recent_view_menu;
+	EggRecentViewGtk *recent_view_toolbar;
 	EggRecentModel   *recent_model;
 	GtkWidget        *file_popup_menu;
 	GtkWidget        *mitem_recents_menu;
+	GtkWidget        *recent_toolbar_menu;
+	GtkAction        *open_action;
 
 	/* drag data */
 
@@ -180,15 +193,20 @@ typedef struct {
 
 	gboolean extracting_dragged_files;
 	gboolean extracting_dragged_files_interrupted;
+	gboolean batch_adding_one_file;
 
 	/* progress dialog data */
 
 	GtkWidget *progress_dialog;
+	GtkWidget *pd_action;
+	GtkWidget *pd_archive;
 	GtkWidget *pd_message;
 	GtkWidget *pd_progress_bar;
 	gboolean   progress_pulse;
 	guint      progress_timeout;  /* Timeout to display the progress dialog. */
 	guint      hide_progress_timeout;  /* Timeout to hide the progress dialog. */
+	FRAction   pd_last_action;
+	char      *pd_last_archive;
 
 	/* batch mode data */
 
@@ -202,10 +220,11 @@ typedef struct {
 
 	gulong     theme_changed_handler_id;
 	gboolean   non_interactive;
+	char      *extract_here_dir;
 } FRWindow;
 
 
-FRWindow * window_new                       ();
+FRWindow * window_new                       (void);
 
 void       window_close                     (FRWindow      *window);
 
@@ -254,6 +273,14 @@ void       window_archive_add_directory     (FRWindow      *window,
 					     FRCompression  compression);
 
 void       window_archive_add_items         (FRWindow      *window,
+					     GList         *dir_list,
+					     const char    *base_dir,
+					     const char    *dest_dir,
+					     gboolean       update,
+					     const char    *password,
+					     FRCompression  compression);
+
+void       window_archive_add_dropped_items (FRWindow      *window,
 					     GList         *item_list,
 					     gboolean       update);
 
@@ -270,6 +297,15 @@ void       window_archive_extract           (FRWindow      *window,
 					     gboolean       junk_paths,
 					     const char    *password);
 
+void       window_archive_extract_here      (FRWindow      *window,
+					     GList         *file_list,
+					     const char    *extract_to_dir,
+					     const char    *base_dir,
+					     gboolean       skip_older,
+					     gboolean       overwrite,
+					     gboolean       junk_paths,
+					     const char    *password);
+
 void       window_archive_close             (FRWindow      *window);
 
 void       window_set_password              (FRWindow      *window,
@@ -301,6 +337,10 @@ GList *    window_get_file_list_selection   (FRWindow    *window,
 					     gboolean     recursive,
 					     gboolean    *has_dirs);
 
+GList *    window_get_file_list_from_path_list (FRWindow *window,
+						GList    *path_list,
+						gboolean *has_dirs);
+
 GList *    window_get_file_list_pattern     (FRWindow    *window,
 					     const char  *pattern);
 
@@ -329,8 +369,12 @@ void       window_view_file                 (FRWindow *window,
 					     char     *file);
 
 void       window_open_files                (FRWindow *window, 
-					     char     *command,
-					     GList    *file_list);
+					     GList    *file_list,
+					     char     *command);
+
+void       window_open_files_with_application (FRWindow *window, 
+					       GList    *file_list,
+					       GnomeVFSMimeApplication *app);
 
 void       window_view_or_open_file         (FRWindow *window, 
 					     char     *file);
@@ -340,7 +384,8 @@ void       window_update_columns_visibility (FRWindow *window);
 void       window_update_history_list       (FRWindow *window);
 
 void       window_set_default_dir           (FRWindow *window,
-					     char     *default_dir);
+					     char     *default_dir,
+					     gboolean  freeze);
 
 void       window_set_open_default_dir      (FRWindow *window,
 					     char     *default_dir);
@@ -397,4 +442,8 @@ void       window_archive__quit               (FRWindow      *window);
 
 void       window_convert_data_free           (FRWindow *window);
 
+gboolean   fr_window_file_list_drag_data_get  (FRWindow         *window,
+					       GList            *path_list,
+					       GtkSelectionData *selection_data);
+
 #endif /* FR_WINDOW_H */
